


######## Principal Component Analysis   ##############

 rm(list=ls())
 RiskFactors = read.csv("RiskFactors.csv")
 RiskFactors[,2] = RiskFactors[,2]*100       #convert to percentage
 RiskFactors[,5:7] = RiskFactors[,5:7]/100       #convert to percentage
 RiskFactors[,8] = RiskFactors[,8]*100        #basis point  

 Dates = as.vector(RiskFactors[,1])
 Dates = strptime(Dates, "%m/%d/%Y") #convert to  POSIXlt (a date class)
 Names = c("Date", "Market risk", "Volatility risk", "Exchange rate risk",
 		"Maturity risk", "Default risk", "Liquidity risk", 
		"Size effect", "Short term rate")

 postscript("Fig61.eps", width=4.4, height=5.5, horizontal=F, pointsize=8)
 par(mfrow = c(4, 2),mar=c(2,3,.5,.5)+0.1, cex=0.8)
 for(i in 2:9)
   plot(Dates, RiskFactors[,i], type="l", ylab=c("",Names[i]), col=4)
 dev.off()
 
 

#######################PCA Analysis #########################
	
	X = RiskFactors[801:2630,2]        #2-23-04 to 2-28-2011
	Y = RiskFactors[801:2630,3:9]  
	tmp = cor(RiskFactors[801:2630,2:9])
    tmp
	resid = resid(lsfit(X, Y))         # take the return of SP500 out
	cor(resid)
        eigen(cor(resid))
  	a = eigen(cor(resid))$vectors[,1]
	a = a / sqrt(apply(resid, 2, var))  	#standardize the variables
        Factor = resid %*% a
        cor(Factor, X)

	postscript("Fig64a.eps", width=4.4, height=3, horizontal=F, pointsize=8)
        par(mfrow = c(1, 1), mar=c(2,2,2,.5)+0.1, cex=0.8)
	plot(Dates[801:2630],   X, 	  type="l", col=2) 
	lines(Dates[801:2630],  2*Factor, lwd=2,    col=4)
        title("Realization of the 1st PC")

        postscript("Fig64b.eps", width=4.4, height=2.5, horizontal=F, pointsize=8)
        par(mfrow = c(1, 2),mar=c(4,3,3,.2)+0.1,  cex=0.8)
        plot(X, Factor, pch=16, main = "Dependence of factors", col=4, 
		xlab="SP 500", ylab= c("", "Other Factors"))
        plot(X, Factor, pch=16, xlim=c(-2,2), main="Dependence of factors",
		 col=4, xlab="SP 500", ylab= c("", "Other Factors"))

        

    #################### Localized PCA with width = 3 mo or 1 y ##########################

	n = length(X)
        width = 63
        Values = NULL        
        for(t in 1:(n-width))
	   {  resid  = resid(lsfit(X[t:(t+width)], Y[t:(t+width),]))
              Eigens = eigen(cor(resid))		#princomp, loadings, summary
              Values = rbind(Values, (Eigens$values)[1:2])
           }
        Values03 = round(Values /ncol(Y), 3)            #percentage of contributions
        
 	postscript("Fig65.eps", width=4.4, height=3, horizontal=F, pointsize=8)
        par(mfrow = c(1, 2), mar=c(2,3,3,.5)+0.1, cex=0.8)
	plot(Dates[800+(width+1):n], Values03[,1], type="l", ylim=c(0,1),
		 col=4, ylab="percent explained")
     	lines(Dates[800+(width+1):n], Values03[,2], type="l", col=2)
        title("Variance explained (w = 3 months)")

	
 	width = 252
        Values = NULL        
        for(t in 1:(n-width))
	   {  Eigens = eigen(cor(Y[t:(t+width),]))		#princomp, loadings, summary
              Values = rbind(Values, (Eigens$values)[1:2])
           }
        Values12 = round(Values /ncol(Y), 3)


	plot(Dates[800+(width+1):n], Values12[,1], type="l", ylim=c(0,1),
		 col=4, ylab="percent explained")
     	lines(Dates[800+(width+1):n], Values12[,2], type="l", col=2)
        title("Variance explained (w = 12 months)")

	
  ################### Financial Crisis Period  #######################
	X1 = RiskFactors[1805:2328,2]        	# 08-01-01 to 09-12-31
	Y1 = RiskFactors[1805:2328,3:9]  
	resid = resid(lsfit(X1, Y1))         	# take the return of SP500 out
	cor(resid)
        eigen(cor(resid))                       #### Result does not change very much

 
  	
    	

  ##################### Prediction power on equities ################
        #### reading Fama-French Portfolio ##############
	FF100 = matrix(scan("100_Portfolios_10x10_Daily.txt", skip=20), ncol=101, byrow=T)
        dates = FF100[,1]
        FF100 = 100*diff(log(FF100[,2:101]))
        FF100 = FF100[474:nrow(FF100),]               #2/24/04 -- 12/31/10
        dates = dates[475:length(dates)]


       
        #### matching the dates ####################        
        D = Dates[802:2589]			      ##Factor Dates  ##+86400 
        D = paste(substr(D, 1, 4), substr(D, 6, 7), substr(D, 9, 10), sep="")
        D = as.numeric(D)                             #convert dates into numeric
	
        ind = rep(0, length(Dates))                   #matching two diff dates
        for(i in 1:length(dates))
           ind[i] = (1:1788)[D == dates[i]]
            
        X2      = X[ind]
        Factor2 = Factor[ind]
        D1      = Dates[802:2630]
        D1      = D1[ind]			      ###the calendar are now dates
        Factors.all = RiskFactors[801:2630,2:9]
        Factors.all = Factors.all[ind,]
         
        ######### cleaning outliers ###############
	resid = resid(lsfit(X2, FF100))
        resid1 = resid                               #dirty outlier 
        for(i in 1:100)                              #cleaning outliers
         { tmp = (resid[,i] > 4)
           resid[tmp,i] = 4
           tmp = (resid[,i] < -4)
           resid[tmp, i] = -4
         }
        FF100   = FF100 - resid1 + resid             #removing outliers by regression

        ######## Computing multiple R^2 by one and two, 7 factors ##########
        resid = resid(lsfit(X2, FF100))				
        Rsq1 = 1-apply(resid, 2, var)/apply(FF100,2,var)
        resid = resid(lsfit(cbind(X2, Factor2), FF100))
        Rsq2 = 1-apply(resid, 2, var)/apply(FF100,2,var)
        resid = resid(lsfit(Factors.all, FF100))
        Rsq3 = 1-apply(resid, 2, var)/apply(FF100,2,var)
	
	postscript("Fig66.eps", width=4.4, height=3, horizontal=F, pointsize=8)
        par(mfrow = c(1, 1), mar=c(2,2,2,.5)+0.1, cex=0.8)
	matplot(cbind(Rsq1, Rsq2, Rsq3), type="l", lwd=2, ylim=c(0,1), col=2:4)
	title("Percent of variability explained by Risk Factors")
        
	
   
     
       #################### Now try moving average of returns ######################
       #################### Use outliers removed data FF100   ######################
       	n = nrow(FF100)
 	FF25 = FF100[1:(n-24),]                 #moving average of 25 days of FF100
	for (i in 2:25)
          {FF25 = FF25 + FF100[i:(n+i-25),]}
        FF25 = FF25/25
 
        Y1 = RiskFactors[801:2630,2:9]           
        Y1 = Y1[ind,] 				# date matched data, dates are the calendar
        RF25 = Y1[1:(n-24),]			# moving average of 25 days
        for (i in 2:25)
          {RF25 = RF25 + Y1[i:(n+i-25),]}
        RF25 = RF25/25

	round(cor(RF25),3)                     #### correlation of 8 factors

                 ########## PCA of residuals ############
	X25 = RF25[,1]
	Y25 = RF25[,2:8]
	resid = resid(lsfit(X25, Y25))          # take the return of SP500 out
	cor(resid)               
        eigen(cor(resid))
  	a = eigen(cor(resid))$vectors[,1]
	a = a / sqrt(apply(resid, 2, var))  	#standardize the variables
        Factor25 = resid %*% a
        cor(Factor25, X25)

        ################ Plot of Factors  ################## 
	postscript("Fig67a.eps", width=4.4, height=3, horizontal=F, pointsize=8)
        par(mfrow = c(1, 1), mar=c(2,2,2,.5)+0.1, cex=0.8)
	plot (D1[25:n],  X25, 	   type="l", col=2) 
	lines(D1[25:n],  Factor25/5, lwd=2,    col=4)
        title("Realization of the 1st PC")

        postscript("Fig67b.eps", width=4.4, height=2.5, horizontal=F, pointsize=8)
        par(mfrow = c(1, 2),mar=c(4,3,3,.2)+0.1,  cex=0.8)
        plot(X25, Factor25, pch=16, main = "Dependence of factors", col=4, 
		xlab="SP 500", ylab= c("", "Other Factors"))
        plot(X25, Factor25, pch=16, xlim=c(-2,2), main="Dependence of factors",
		 col=4, xlab="SP 500", ylab= c("", "Other Factors"))

       ############# compute Rsq using 1 factor, 2 factor, 7 factors ###########
	resid = resid(lsfit(X25, FF25))				
        Rsq1 = 1-apply(resid, 2, var)/apply(FF25,2,var)
        resid = resid(lsfit(cbind(X25, Factor25), FF25))
        Rsq2 = 1-apply(resid, 2, var)/apply(FF25,2,var)
        resid = resid(lsfit(RF25, FF25))
        Rsq3 = 1-apply(resid, 2, var)/apply(FF25,2,var)

        postscript("Fig68.eps", width=4.4, height=3, horizontal=F, pointsize=8)
        par(mfrow = c(1, 1), mar=c(2,2,2,.5)+0.1, cex=0.8)
	matplot(cbind(Rsq1, Rsq2, Rsq3), type="l", lwd=2, ylim=c(0,1), col=2:4)
	title("Percent of variability explained by Risk Factors")
        
	

       
       

   
       

        

        

     



















  
  
  
  
  
  
  
  
  
  
  

